function pointSourceColocForJosephCabral(movieData)
% POINTSOURCESCORING Performs post-processing to improve masks for an input
% movie, and scores point sources according to improved masks
% 
% movieData = pointSourceScoring(movieData)
% 
% movieData = pointSourceScoring(movieData,paramsIn)
% 
% Input:
% 
%   movieData - The movieData object describing the movie, as created using
%   setupMovieDataGUI.m
% 
%   paramsIn - Structure with inputs for optional parameters. The
%   parameters should be stored as fields in the structure, with the field
%   names and possible values as described below:
% 
%   Possible Parameter Structure Field Names:
%       ('FieldName' -> possible values)
%
%       ('ChannelIndex'-> Positive integer scalar or vector) 
%       The integer indices of the channel(s) to perform mask refinement
%       on. This index corresponds to the channel's location in
%       the array movieData.channels_. If not input, the user
%       will be asked to select from the available channels
%
%       ('SegProcessIndex' -> Positive integer scalar or vector) Optional.
%       This specifies MaskProcess(s) to use masks from by its
%       index in the array movieData.processes_; For each channel, masks
%       will be used from the last process specified which has valid masks
%       for that channel. That is if SegProcessIndex = [1 4] and both
%       processes 1 and 4 have masks for a given channel, then the masks
%       from process 4 will be used. If not input, and multiple
%       MaskProcesses are present, the user will be asked to select
%       one, unless batch mode is enabled in which case an error will be
%       generated.
%
%       ('OutputDirectory' -> character string)
%       Optional. A character string specifying the directory to save the
%       masks to. Masks for different channels will be saved as
%       sub-directories of this directory.
%       If not input, the masks will be saved to the same directory as the
%       movieData, in a sub-directory called "masks"
%
%       ('BatchMode' -> True/False)
%       If this option value is set to true, all graphical output and user
%       interaction is suppressed.
% 
% 
% Marcelo Cicconet, Mar 2017
% Adapted from pointSourceScoring
%

%% ----------- Input --------- %%

ip = inputParser;
ip.addRequired('movieData',@(x)(isa(x,'MovieData')));
ip.parse(movieData);
                                                                
iProc = movieData.getProcessIndex('PointSourceColocForJosephCabralProcess',1,0);

%If the process doesn't exist, create it
if isempty(iProc)
    error('No PointSourceColocForJosephCabralProcess in input movieData! please create the process and use the process.run method to run this function!')
end

segProc = movieData.processes_{iProc};
%Parse input, store in parameter structure
p = parseProcessParams(segProc);

maskProcessIndex = movieData.getProcessIndex('FilterMaskPropsProcess',Inf,0);
ptSrcProcessIndex = movieData.getProcessIndex('LoGPointSourceDetectionProcess',Inf,0);

if isempty(maskProcessIndex)
    error('This function requires that the input movie has already been segmented and that a valid FilterMaskPropsProcess be specified!')
end
if isempty(ptSrcProcessIndex) 
    error('This function requires that a valid LoGPointSourceDetectionProcess be specified!')
end

%% ----------- Init ----------- %%

% mask channel
for i = 1:length(movieData.processes_{maskProcessIndex}.outFilePaths_)
    if ~isempty(movieData.processes_{maskProcessIndex}.outFilePaths_{i})
        maskChan = i;
        break;
    end
end
% maskChan = 1;
inMaskDir = movieData.processes_{maskProcessIndex}.outFilePaths_(maskChan);
inMaskDir = inMaskDir{1};
outMaskDir = [p.OutputDirectory filesep 'mask' num2str(maskChan)];
mkClrDir(outMaskDir)

% point source channels
psp = movieData.processes_{ptSrcProcessIndex};
ptSrcIndexMask = zeros(1,length(psp.outFilePaths_));
for i = 1:length(psp.outFilePaths_)
    ptSrcIndexMask(i) = ~isempty(psp.outFilePaths_{i});
end
ptSrcIndices = [];
for i = 1:length(ptSrcIndexMask)
    if ptSrcIndexMask(i)
        ptSrcIndices = [ptSrcIndices i];
    end
end

% reference point source channel
refPtSrcChan = p.ChannelIndex;

% disp(ptSrcIndices)
% disp(refPtSrcChan)
% pause

movieData.processes_{iProc}.setOutMaskPath(maskChan,outMaskDir);
movieData.processes_{iProc}.setInMaskPath(maskChan,inMaskDir);

inPtSrcPath = cell(1,length(ptSrcIndexMask));
for j = 1:length(ptSrcIndexMask)
    inPtSrcPath{j} = psp.outFilePaths_{j};
end
nonRefPtSrcChans = ptSrcIndices;
nonRefPtSrcChans(ptSrcIndices == refPtSrcChan) = [];
% ptSrcIndices: all indices of point sources
% refPtSrcChan: reference index
% nonRefPtSrcChans: all other indices

%% ---------------- Point Source Scoring --------------- %%

% mask
inMaskPath = [inMaskDir filesep 'Filtered_mask_1.tif'];
maskIn = imread(inMaskPath);
cc = bwconncomp(maskIn);

sOb = regionprops(cc,'Centroid','MajorAxisLength','MinorAxisLength','PixelIdxList','Area'); % statistics of objects (nuclei)
nOb = length(sOb); % number of objects (nuclei)
centroids = cat(1,sOb.Centroid);
radii = mean([cat(1,sOb.MajorAxisLength) cat(1,sOb.MinorAxisLength)],2)/2;
areas = cat(1,sOb.Area);

% count point sources
scores = zeros(nOb,5+length(ptSrcIndices)+2*length(nonRefPtSrcChans)); % object ID | center x | center y | radius | area | counts...
obIDs = zeros(nOb,1);
for i = 1:nOb
    obIDs(i) = maskIn(sOb(i).PixelIdxList(1));
end
scores(:,1) = obIDs;
scores(:,2:3) = round(centroids);
scores(:,4) = round(radii);
scores(:,5) = round(areas);

maxNumObj = max(max(maskIn));
nChan = length(ptSrcIndices);
for iChan = 1:nChan    
    fprintf('scoring channel %d/%d\n', iChan, nChan);
    load(inPtSrcPath{ptSrcIndices(iChan)}) % loads movieInfo
    psx = round(movieInfo.xCoord);
    psy = round(movieInfo.yCoord);
    amp = movieInfo.amp;
    
    psCount = zeros(maxNumObj,1);
    for i = 1:length(psx)
        obIdx = maskIn(psy(i),psx(i));
        if obIdx > 0
            psCount(obIdx) = psCount(obIdx)+(amp(i) > 0);
        end
    end
    
    c0 = 5+iChan;
    scores(:,c0) = psCount(obIDs);
end

% colocalization with respect to reference channel
load(inPtSrcPath{refPtSrcChan}) % loads movieInfo (for reference channel)
psx = round(movieInfo.xCoord);
psy = round(movieInfo.yCoord);
amp = movieInfo.amp;
maskRefPtSrc = zeros(size(maskIn));
for i = 1:length(psx)
    maskRefPtSrc(psy(i),psx(i)) = (amp(i) > 0);
end
% imwrite(maskIn > 0,[p.OutputDirectory filesep 'mask_nuclei.png']);
dlMaskRefPtSrc = imdilate(maskRefPtSrc,strel('disk',p.ProximityThreshold,0));
% imwrite(dlMaskRefPtSrc,[p.OutputDirectory filesep sprintf('mask_ref_pt_src_%d.png',refPtSrcChan)]);
nChan = length(nonRefPtSrcChans);
for iChan = 1:nChan
    fprintf('coloc. analysis, channel %d/%d\n', iChan, nChan);
    load(inPtSrcPath{nonRefPtSrcChans(iChan)}) % loads movieInfo
    psx = round(movieInfo.xCoord);
    psy = round(movieInfo.yCoord);
    amp = movieInfo.amp;
    
%     maskNonRefPtSrc = zeros(size(maskIn,1),size(maskIn,2),3);
%     maskNonRefPtSrc(:,:,2) = dlMaskRefPtSrc;
%     maskNonRefPtSrcColoc = zeros(size(maskIn));
    psCount = zeros(maxNumObj,1);
    for i = 1:length(psx)
%         maskNonRefPtSrc(psy(i),psx(i),:) = 0;
%         maskNonRefPtSrc(psy(i),psx(i),1) = (amp(i) > 0);
        obIdx = maskIn(psy(i),psx(i));
        if obIdx > 0 && dlMaskRefPtSrc(psy(i),psx(i)) > 0
            psCount(obIdx) = psCount(obIdx)+(amp(i) > 0);
%             maskNonRefPtSrcColoc(psy(i),psx(i)) = (amp(i) > 0);
        end
    end
    
%     imwrite(maskNonRefPtSrc,[p.OutputDirectory filesep sprintf('mask_non_ref_pt_src_%d.png',nonRefPtSrcChans(iChan))]);
%     imwrite(maskNonRefPtSrcColoc,[p.OutputDirectory filesep sprintf('mask_non_ref_pt_src_%d_coloc.png',nonRefPtSrcChans(iChan))]);
    
    c0 = 5+length(ptSrcIndices)+iChan;
    scores(:,c0) = psCount(obIDs);
end

varNames = cell(1,5+nChan);
varNames{1} = 'obj_id';
varNames{2} = 'cent_x';
varNames{3} = 'cent_y';
varNames{4} = 'radius';
varNames{5} = 'area';
colIndex = 0;
colCountNonRefPtSrcChans = zeros(1,length(nonRefPtSrcChans));
for iChan = 1:length(ptSrcIndices)
    c0 = 5+iChan;
    varNames{c0} = sprintf('nps_mask_ch_%d',ptSrcIndices(iChan));
    col = find(nonRefPtSrcChans == ptSrcIndices(iChan),1);
    if ~isempty(col)
        colIndex = colIndex+1;
        colCountNonRefPtSrcChans(colIndex) = 5+iChan;
    end
end
for iChan = 1:length(nonRefPtSrcChans)
    c00 = 5+length(ptSrcIndices)+iChan;
    varNames{c00} = sprintf('nps_%d_coloc_wrt_%d',nonRefPtSrcChans(iChan),refPtSrcChan);
    c01 = 5+length(ptSrcIndices)+length(nonRefPtSrcChans)+iChan;
    varNames{c01} = sprintf('pct_%d_coloc_wrt_%d',nonRefPtSrcChans(iChan),refPtSrcChan);
%     scores(:,c01) = scores(:,c00)./scores(:,colCountNonRefPtSrcChans(iChan));
end
sscores = sum(scores);
sscores(1:4) = NaN;
scores = [sscores; scores];
for iChan = 1:length(nonRefPtSrcChans)
    c00 = 5+length(ptSrcIndices)+iChan;
    c01 = 5+length(ptSrcIndices)+length(nonRefPtSrcChans)+iChan;
    scores(:,c01) = 100*scores(:,c00)./scores(:,colCountNonRefPtSrcChans(iChan));
end
T = array2table(scores,'VariableNames',varNames);
writetable(T,[p.OutputDirectory filesep 'scores.csv']);

outMaskPath = [outMaskDir filesep 'Scoring_mask1.tif']; % expanded mask
imwrite(cast(maskIn,minIntClass(maxNumObj)),outMaskPath);

%% ------ Finalize ------ %%


%Store parameters/settings in movieData structure

movieData.processes_{iProc}.setDateTime;
movieData.processes_{iProc}.setMaxIndex(maxNumObj);
movieData.save; %Save the new movieData to disk

disp('Finished scoring point sources!')

