function simplePointSourceScoring(movieData)
% POINTSOURCESCORING Performs post-processing to improve masks for an input
% movie, and scores point sources according to improved masks
% 
% movieData = pointSourceScoring(movieData)
% 
% movieData = pointSourceScoring(movieData,paramsIn)
% 
% Input:
% 
%   movieData - The movieData object describing the movie, as created using
%   setupMovieDataGUI.m
% 
%   paramsIn - Structure with inputs for optional parameters. The
%   parameters should be stored as fields in the structure, with the field
%   names and possible values as described below:
% 
%   Possible Parameter Structure Field Names:
%       ('FieldName' -> possible values)
%
%       ('ChannelIndex'-> Positive integer scalar or vector) 
%       The integer indices of the channel(s) to perform mask refinement
%       on. This index corresponds to the channel's location in
%       the array movieData.channels_. If not input, the user
%       will be asked to select from the available channels
%
%       ('SegProcessIndex' -> Positive integer scalar or vector) Optional.
%       This specifies MaskProcess(s) to use masks from by its
%       index in the array movieData.processes_; For each channel, masks
%       will be used from the last process specified which has valid masks
%       for that channel. That is if SegProcessIndex = [1 4] and both
%       processes 1 and 4 have masks for a given channel, then the masks
%       from process 4 will be used. If not input, and multiple
%       MaskProcesses are present, the user will be asked to select
%       one, unless batch mode is enabled in which case an error will be
%       generated.
%
%       ('OutputDirectory' -> character string)
%       Optional. A character string specifying the directory to save the
%       masks to. Masks for different channels will be saved as
%       sub-directories of this directory.
%       If not input, the masks will be saved to the same directory as the
%       movieData, in a sub-directory called "masks"
%
%       ('BatchMode' -> True/False)
%       If this option value is set to true, all graphical output and user
%       interaction is suppressed.
% 
% 
% Marcelo Cicconet, Mar 2017
% Adapted from pointSourceScoring
%

%% ----------- Input --------- %%

ip = inputParser;
ip.addRequired('movieData',@(x)(isa(x,'MovieData')));
ip.parse(movieData);
                                                                
iProc = movieData.getProcessIndex('SimplePointSourceScoringProcess',1,0);

%If the process doesn't exist, create it
if isempty(iProc)
    error('No SimplePointSourceScoringProcess in input movieData! please create the process and use the process.run method to run this function!')
end

segProc = movieData.processes_{iProc};
%Parse input, store in parameter structure
p = parseProcessParams(segProc);

maskProcessIndex = movieData.getProcessIndex('FilterMaskPropsProcess',Inf,0);
ptSrcProcessIndex = movieData.getProcessIndex('LoGPointSourceDetectionProcess',Inf,0);

if isempty(maskProcessIndex)
    error('This function requires that the input movie has already been segmented and that a valid FilterMaskPropsProcess be specified!')
end
if isempty(ptSrcProcessIndex) 
    error('This function requires that a valid LoGPointSourceDetectionProcess be specified!')
end

%% ----------- Init ----------- %%

% mask channel
maskChan = 1;
inMaskDir = movieData.processes_{maskProcessIndex}.outFilePaths_(maskChan);
inMaskDir = inMaskDir{1};
outMaskDir = [p.OutputDirectory filesep 'mask' num2str(maskChan)];
mkClrDir(outMaskDir)

movieData.processes_{iProc}.setOutMaskPath(maskChan,outMaskDir);
movieData.processes_{iProc}.setInMaskPath(maskChan,inMaskDir);

% point source channels
nChan = length(p.ChannelIndex);

% nImages = movieData.nFrames_;
% nImTot = nImages * nChan;

inPtSrcPath = cell(1,nChan);
% outPtSrcDir = cell(1,nChan);
% outPtSrcPath = cell(1,nChan);
for j = 1:nChan
    
    %Get the most recent seg process with masks for this channel
    inPtSrcPath{j} = movieData.processes_{ptSrcProcessIndex}.outFilePaths_(p.ChannelIndex(j));
    
    %Create string for current directory
%     outPtSrcDir{j} = [p.OutputDirectory filesep 'score' num2str(p.ChannelIndex(j))];
%     outPtSrcPath{j} = [outPtSrcDir{j} '/PtSrc.mat'];
        
    %Check/create directory
%     mkClrDir(outPtSrcDir{j})
end
% movieData.processes_{iProc}.setOutFilePaths(outPtSrcPath);
% movieData.processes_{iProc}.setInFilePaths(inPtSrcPath);

%% ---------------- Point Source Scoring --------------- %%

% if ~p.BatchMode && feature('ShowFigureWindows')
%     wtBar = waitbar(0,'Please wait...');
% else
%     wtBar = -1;
% end        

% mask
inMaskPath = [inMaskDir filesep 'Filtered_mask_' num2str(maskChan) '.tif'];
maskIn = imread(inMaskPath);
cc = bwconncomp(maskIn);

sOb = regionprops(cc,'Centroid','MajorAxisLength','MinorAxisLength','PixelIdxList','Area'); % statistics of objects (nuclei)
nOb = length(sOb); % number of objects (nuclei)
centroids = cat(1,sOb.Centroid);
radii = mean([cat(1,sOb.MajorAxisLength) cat(1,sOb.MinorAxisLength)],2)/2;
areas = cat(1,sOb.Area);

% score
scores = zeros(nOb,5+nChan); % object ID | center x | center y | radius | area | nChan*(n. puncta in region)
obIDs = zeros(nOb,1);
for i = 1:nOb
    obIDs(i) = maskIn(sOb(i).PixelIdxList(1));
end
scores(:,1) = obIDs;
scores(:,2:3) = round(centroids);
scores(:,4) = round(radii);
scores(:,5) = round(areas);

maxNumObj = max(max(maskIn));

% summary images
% NucleiContours = double(bwmorph(maskIn,'remove'));
for iChan = 1:nChan    
%     SummaryImage = zeros(size(maskIn,1),size(maskIn,2),3);
%     SummaryImage(:,:,3) = normalize(double(movieData.channels_(p.ChannelIndex(1)).loadImage(1)));
%     SummaryImage(:,:,2) = normalize(double(movieData.channels_(p.ChannelIndex(iChan)).loadImage(1)));
%     SummaryImage(:,:,1) = NucleiContours;
    
    fprintf('scoring channel %d/%d\n', iChan, nChan);
    load(inPtSrcPath{iChan}{1}) % loads movieInfo
    psx = round(movieInfo.xCoord);
    psy = round(movieInfo.yCoord);
    amp = movieInfo.amp;
    
    psCount = zeros(maxNumObj,1);
    for i = 1:length(psx)
%         SummaryImage(psy(i),psx(i),:) = reshape([0 0 1],[1 1 3]);
        obIdx = maskIn(psy(i),psx(i));
        if obIdx > 0
            psCount(obIdx) = psCount(obIdx)+(amp(i) > 0);
        end
    end
    
    c0 = 5+iChan;
    scores(:,c0) = psCount(obIDs);
    
    % point source heat map visualization
    lbs = cell(1,nOb);
    for i = 1:nOb
%         SummaryImage = insertText(SummaryImage,centroids(i,:),sprintf('%d',obIDs(i)),'FontSize',16,'BoxOpacity',0,'TextColor','white');
        
        lbs{i} = sprintf('%d',obIDs(i));
    end
    I = zeros(size(maskIn)); % original regions
    for iOb = 1:nOb
        I(sOb(iOb).PixelIdxList) = psCount(obIDs(iOb));
    end
    writeFloatImageToTiffFile(single(I),[p.OutputDirectory filesep sprintf('PointSourceHeatMap_Channel%d.tif',p.ChannelIndex(iChan))]);
    
%     imwrite(SummaryImage(:,:,:,iChan), [p.OutputDirectory filesep sprintf('SummaryImage_Channel%d.png',p.ChannelIndex(iChan))]);
end

varNames = cell(1,5+nChan);
varNames{1} = 'obj_id';
varNames{2} = 'cent_x';
varNames{3} = 'cent_y';
varNames{4} = 'radius';
varNames{5} = 'area';
for iChan = 1:nChan
    c0 = 5+iChan;
    varNames{c0} = sprintf('nps_mask_ch_%d',p.ChannelIndex(iChan));
end
T = array2table(scores,'VariableNames',varNames);
writetable(T,[p.OutputDirectory filesep 'scores.csv']);

outMaskPath = [outMaskDir filesep 'Scoring_mask' num2str(maskChan) '.tif']; % expanded mask
imwrite(cast(maskIn,minIntClass(maxNumObj)),outMaskPath);

% if ishandle(wtBar)
%     close(wtBar)
% end


%% ------ Finalize ------ %%


%Store parameters/settings in movieData structure

movieData.processes_{iProc}.setDateTime;
movieData.processes_{iProc}.setMaxIndex(maxNumObj);
movieData.save; %Save the new movieData to disk

disp('Finished scoring point sources!')

