function unsupervisedSegmentation(movieData)
% segments and labels input movie
%
% Marcelo Cicconet
% Mar 2017

%% --------------- Parameters ---------- %%

pString = 'mask_'; %Prefix for saving masks to file
dName = 'channel ';%Sub-directory name for per-channel masks

%% ------------------ Input ---------------- %%

ip = inputParser;
ip.addRequired('movieData',@(x)(isa(x,'MovieData')));
ip.parse(movieData);

%Get the indices of any previous speckle detection processes                                                                     
iProc = movieData.getProcessIndex('UnsupervisedSegmentationProcess',1,0);

%If the process doesn't exist, create it
if isempty(iProc)
    error('No UnsupervisedSegmentationProcess in input movieData! please create the process and use the process.run method to run this function!')
end

segProc = movieData.processes_{iProc};
%Parse input, store in parameter structure
p = parseProcessParams(segProc);


%% --------------- Init -------------- %%

nChanSeg = numel(p.ChannelIndex);


% Set up the input directories (input images)
inFilePaths = cell(1,numel(movieData.channels_));
for i = p.ChannelIndex    
    inFilePaths{1,i} = movieData.getChannelPaths{i};    
end
segProc.setInFilePaths(inFilePaths);

%Set up the mask directories as sub-directories of the output directory
for j = 1:nChanSeg
    
    %Create string for current directory
    currDir = [p.OutputDirectory filesep dName num2str(p.ChannelIndex(j))];    
    %Save this in the process object
    segProc.setOutMaskPath(p.ChannelIndex(j),currDir);
   
    %Check/create directory
    mkClrDir(currDir)               
end

nImages = movieData.nFrames_;   
nImTot = nImages * nChanSeg;
imSize = movieData.imSize_;

fStr = ['%0' num2str(floor(log10(nImages))) + 1 '.0f'];%Format string for zero-padding mask file anmes


%Get mask and image directories
maskDirs  = segProc.outFilePaths_(p.ChannelIndex);

%TEMP - check for and load thresholding/mask refinement 
% masks = false([imSize, nImages nChanSeg]);
masks = zeros([imSize, nImages nChanSeg]);

%Get the ROI mask (if not an ROI, this will be all true)
roiMask = movieData.getROIMask;

%% ------------ Segmentation -------------- %%

% if ~p.BatchMode && feature('ShowFigureWindows')
%     wtBar = waitbar(0,['Segmenting channel ' num2str(p.ChannelIndex(1)) ' ...']);        
% else
%     wtBar = -1;
% end        

maxNumObjects = 0;
for iChan = 1:nChanSeg
    for iImage = 1:nImages
        % ---------- Initial Segmentation ------------- %
        
        %Load the current image        
        currImage = movieData.channels_(p.ChannelIndex(iChan)).loadImage(iImage);
        
        % random forest segmentation
%         if ~p.BatchMode && feature('ShowFigureWindows')
%             ForegroundMask = rfsegment(rfModel,currImage,p.WatershedPostProcFlag,p.WatershedParams,'DisplayProgressBar');
%         else
%             ForegroundMask = rfsegment(rfModel,currImage,p.WatershedPostProcFlag,p.WatershedParams);
%         end

%         disp(p.sigma)
%         disp(p.hmin)

%         ForegroundMask = imbinarize(currImage);
        ForegroundMask = imbinarize(normalize(filterGauss2D(currImage,p.sigma)));
        ForegroundMask = imopen(ForegroundMask, strel('disk',3,0));
        
        ForegroundSurface = imhmin(1-normalize(bwdist(not(ForegroundMask))),p.hmin);
        ForegroundSurface(not(ForegroundMask)) = -Inf;
        ForegroundSurface(:,1) = -Inf; ForegroundSurface(:,end) = -Inf; ForegroundSurface(1,:) = -Inf; ForegroundSurface(end,:) = -Inf;
        L = watershed(ForegroundSurface);
%         figure, imshow(L >= 2)
        ForegroundMask = imfill(L >= 2,'holes');
        
        currMask = bwlabel(ForegroundMask); % we want one label per foreground object
        
        masks(:,:,iImage,iChan) = currMask;
        cc = bwconncomp(currMask);
        if cc.NumObjects > maxNumObjects
            maxNumObjects = cc.NumObjects;
        end
         
%         if ishandle(wtBar) && mod(iImage,5)
%             %Update the waitbar occasionally to minimize slowdown
%             waitbar((iImage + (iChan-1)*nImages) / nImTot,wtBar)
%         end
    end    
end

%% ---------- ROI Selection ----------- %%
%Applies ROI (if selected - the ROI mask is all 1s otherwise)

for iChan = 1:nChanSeg
    masks(:,:,:,iChan) = masks(:,:,:,iChan).*roiMask; 
end


%% ------------- Output ----------------- %%
%Writes the final masks to disk

for iChan = 1:nChanSeg
    
%     if ishandle(wtBar)        
%         waitbar(1/2,wtBar,['Saving mask to disk for channel ' num2str(p.ChannelIndex(iChan)) ' ...']);        
%     end   
    
    for iImage = 1:nImages

        %write the mask to file                    
        imwrite(cast(masks(:,:,iImage,iChan),minIntClass(maxNumObjects)),[maskDirs{iChan} filesep pString num2str(iImage,fStr) '.tif']);
        
%         if ishandle(wtBar) && mod(iImage,5)
%             %Update the waitbar occasionally to minimize slowdown
%             waitbar((iImage + (iChan-1)*nImages) / nImTot,wtBar)
%         end

    end
end

%Store the maximum number of objects in the process class
if maxNumObjects > 0
    segProc.setMaxIndex(maxNumObjects);
else
    segProc.setMaxIndex(1);
end


% if ishandle(wtBar)
%     close(wtBar)
% end

end