classdef SpecialSegProcess < SegmentationProcess
    %SPECIALSEGPROCESS is a segmentation process for any customized
    %segmentation that needs simple numerical input. Any child segmentation
    %process may inheret from this process class and share the GUI and
    %moviedata processing function, with some minor modifacations in
    %funParams and name changes. To use this process, an SWE may directly insert
    %SpecialSegProcess into a new Package classdef, in which case user may
    %need to select which sepecific segmentation needs to be used. Or an
    %SWE may insert any specific subclasses so that all parameters are
    %defaulted correctly, and disable the method selection. 

    methods (Access = public)
        function obj = SpecialSegProcess(owner,varargin)
            if nargin == 0
                super_args = {};
            else
                % Input check
                ip = inputParser;
                ip.addRequired('owner',@(x) isa(x,'MovieData'));
                ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
                ip.addOptional('funParams',[],@isstruct);
                ip.addOptional('funName',[]);
                ip.parse(owner,varargin{:});
                outputDir = ip.Results.outputDir;
                funParams = ip.Results.funParams;
                funName = ip.Results.funName;
                % Define arguments for superclass constructor
                super_args{1} = owner;
                super_args{2} = SpecialSegProcess.getName;
                if isempty(funName)
                    super_args{3} = @specialSegmentation;
                else
                    super_args{3} = funName;
                end
                if isempty(funParams)
                    funParams = SpecialSegProcess.getDefaultParams(owner,outputDir);
                end
                super_args{4} = funParams;
            end
            
            obj = obj@SegmentationProcess(super_args{:});
        end
        
    end
    methods (Static)
        function name = getName()
            name = 'Special Segmentation';
        end
        function h = GUI()
            h= @specialSegProcessGUI;
        end
        function funParams = getDefaultParams(owner,varargin)
            % Input check
            ip=inputParser;
            ip.addRequired('owner',@(x) isa(x,'MovieData'));
            ip.addOptional('outputDir',owner.outputDirectory_,@ischar);
            ip.addOptional('methodSelection',1,@isnumeric);
            ip.parse(owner, varargin{:})
            outputDir=ip.Results.outputDir;
            methodSelection = ip.Results.methodSelection;
            % Set default parameters
            [~, funParams] = SpecialSegProcess.getParamnames('methodSelection',methodSelection);
            funParams.ChannelIndex = 1:numel(owner.channels_);
            funParams.OutputDirectory = [outputDir  filesep 'masks_special'];
            funParams.methodSelection = 1;    
            funParams.ProcessIndex = []; %default to use raw image.
        end
        function methods = getMethods(varargin)
            SpecialSegMethods(1).name = 'Filter Paper Segmentation';
            SpecialSegMethods(2).name = 'Urine Spot Segmentation';
%             NucSegMethods(2).name = 'Nuclei Segmentation by Scale Map';
%             NucSegMethods(2).func = @detectNucleiByScaleMap;
%             NucSegMethods(3).name = 'Nuclei Segmentation for Mario';
%             NucSegMethods(3).func = @detectNucleiByEdgeMario;
%             NucSegMethods(4).name = 'Nuclei Segmentation by response';
%             NucSegMethods(4).func = @edgeSeg;
            ip=inputParser;
            ip.addOptional('index',1:length(SpecialSegMethods),@isvector); %returns all of the methods by default.
            ip.parse(varargin{:});
            index = ip.Results.index;
            methods=SpecialSegMethods(index);
        end
        function [paramNames, funParams] = getParamnames(varargin)
            %% put all parameter names here
            method_paramsN{1} = {'thPaper', 'Scale', 'FilterPaperLength_mm'};
            method_params{1} = {-1, 0.7071, 277};
            method_paramsN{2} = {'maxRectangularBoxBorderPxRatio', 'minSpotArea_Pixel', 'volume_Ul_Per_MMSQ'};
            method_params{2} = {0.25, 9, 0.283};
            %% put new parameters for new methods in
            % method_paramsN{3} = {...}; method_params{3} = {...};
            ip = inputParser;
            ip.addOptional('methodSelection', 1, @isnumeric);
            ip.parse(varargin{:});
            methodSelection = ip.Results.methodSelection;
            paramNames = method_paramsN{methodSelection};
            for i = 1:length(paramNames)
                funParams.(paramNames{i}) = method_params{methodSelection}{i};
            end
        end
    end
end
