function movieData = filterMovieMaskProperties(movieData, varargin)
% CALCMOVIEMASKPROPERTIES calculates objects in a movie masks
%
% calcMovieMaskProperties
%
% SYNOPSIS calcMovieMaskProperties(movieData, paramsIn)
%
% INPUT
% movieData
% MaskPropsCalc - an object which saves all property and calculation
% functions and options.

pString = 'Filtered_'; %Prefix for saving masks to file
dName = 'Filtered_Mask_for_Channel_';

%% ----------- Input ----------- %%

%Check input
ip = inputParser;
ip.CaseSensitive = false;
ip.addRequired('movieData', @(x) isa(x,'MovieData'));
ip.addOptional('CurrProcess',[], @(x) isa(x, 'FilterMaskPropsProcess'));
ip.addOptional('paramsIn',[], @isstruct);

ip.parse(movieData,varargin{:});
paramsIn=ip.Results.paramsIn;
CurrProcess=ip.Results.CurrProcess;

%Get the indices of any previous speckle detection processes
iProcCal = movieData.getProcessIndex('CalcMaskPropsProcess',1,0);
iProc = [];
if ~isempty(CurrProcess)
iProc = movieData.getProcessIndex(CurrProcess,1,0);
end
% bug!!! when two filtering processes are done all at once... the first one
% cannot load.


%If the process doesn't exist, create it
if isempty(iProc)
    warndlg('It is not recommended to used this process without GUI.');                                                                                                 
end
if isempty(iProcCal)
    warndlg('CalcMaskPropsProcess has to be performed prior to filtering.');                                                                                                 
end
FilterMPProcess = movieData.processes_{iProc};
CalcMPProcess = movieData.processes_{iProcCal};
%Parse input, store in parameter structure
p = parseProcessParams(FilterMPProcess,paramsIn);

%% load in masks
maskUsed = CalcMPProcess.funParams_.maskUsed;
maskprocessforuse = movieData.processes_{maskUsed};
nChanThresh = length(p.ChannelIndex);
nImages = movieData.nFrames_;
nImTot = nImages * nChanThresh;

%Set up the input and mask directories
inMaskDir = cell(1,nChanThresh);
outMaskDir = cell(1,nChanThresh);
propsNames = cell(1,nChanThresh);
for j = 1:nChanThresh;    
    maskNames(j) = maskprocessforuse.getOutMaskFileNames(p.ChannelIndex(j));
    inMaskDir(j) = maskprocessforuse.outFilePaths_(p.ChannelIndex(j));
    
    %Create string for current directory
    currDir = [p.OutputDirectory filesep dName num2str(p.ChannelIndex(j))];
        
    %Check/create directory
    mkClrDir(currDir);
    %Save this in the process object
    movieData.processes_{iProc}.setOutMaskPath(p.ChannelIndex(j),currDir);
    movieData.processes_{iProc}.setInMaskPath(p.ChannelIndex(j),inMaskDir{j});
    outMaskDir{j} = currDir;
end


%% ---------------- Property Calculations --------------- %%
% if ~p.BatchMode && feature('ShowFigureWindows')
%     wtBar = waitbar(0,['Please wait, computing mask properties for channel ' num2str(p.ChannelIndex(1)) ' ...']);     
% else
    wtBar = -1;
% end       
for iChan = 1:nChanThresh

    disp(['Calculating properties for channel ' num2str(p.ChannelIndex(iChan)) '...']);
    disp(['Using masks from ' inMaskDir{iChan} ', storing results in ' outMaskDir{iChan}])
    MPropsC = CalcMPProcess.loadChannelOutput(iChan);
    
    for iImage = 1:nImages;
        if ishandle(wtBar)
            waitbar(((iChan-1)*nImages+iImage) / nImTot,wtBar,['Please wait, filtering properties for channel ' num2str(p.ChannelIndex(iChan)) ' ...']);
        end
        %Load the mask for this frame/channel
        % currMask = maskprocessforuse.loadChannelOutput(iChan, iImage);%imread([inMaskDir{iChan} filesep maskNames{iChan}{iImage}]);
        currMask = maskprocessforuse.loadChannelOutput(p.ChannelIndex(iChan), iImage);%imread([inMaskDir{iChan} filesep maskNames{iChan}{iImage}]);
        % Matrix Compare
        %get matrix
        currMatrix = MPropsC.Results(iImage).thresholdableMatrix;
        if isempty(currMatrix)
            imwrite(currMask,[outMaskDir{iChan} filesep pString maskNames{iChan}{iImage}]);
        else
            % get upper and lower limits
            rangelimits = p.rangeSettingMatrix{iChan};
            upperlimits = rangelimits(:,2)';
            lowerlimits = rangelimits(:,1)';
            % get logic array
            if size(currMatrix, 2) ~= length(lowerlimits)
                errordlg('Intensity Properties have been caculated for multiple channels, please reset the filtering criteria!');
                return;
            end
            [~, AndLogic, ~] = matrixCompare(currMatrix, lowerlimits, upperlimits);
            
            % output mask and imwrite.
            if islogical(currMask);
                currMask = bwlabel(currMask);
            end
            filteredMask = filterObjects(currMask, AndLogic);
            filteredMask = cast(filteredMask,minIntClass(max(filteredMask(:))));
            imwrite(filteredMask,[outMaskDir{iChan} filesep pString maskNames{iChan}{iImage}]);
            maxIndex = max(filteredMask(:));
            maxIndexCurr = FilterMPProcess.getMaxIndex;
            if isempty(maxIndexCurr)
                maxIndexCurr = 1;
            end
            if maxIndex <1
                maxIndex = 1;
            end
            FilterMPProcess.setMaxIndex(max(uint16(maxIndex),uint16(maxIndexCurr)));
        end
    end
    %save limits range
    csvwrite([p.OutputDirectory filesep 'PropsRangeLimits.csv'], rangelimits); %% put the names in or put them in a text file.
end

if ishandle(wtBar)
    close(wtBar)
end


%% ------ Finalize ------ %%

%save thresholdable filename;

%Store parameters/settings in movieData structure

movieData.processes_{iProc}.setDateTime;
movieData.save; %Save the new movieData to disk

disp('Finished filtering masks!');
