function movieData = specialSegmentation(movieData, paramsIn)
% SPECIALSEGMENTATION is a sample movieData processing function for
% SpecialSegProcess class, it may be used when only one SpecialSegProcess
% is used in one package, or in one moviedata. If more than one
% SpecialSegProcess are used, each child SpecialSegProcess needs a separate
% movieData processing function.

%% ----- Parameters ----- %%

pString = 'Special_Mask_'; %Prefix for saving masks to file
dName = 'masks_for_channel_';%String for naming the mask directories for each channel

%% ----------- Input ----------- %%
%Check that input object is a valid moviedata TEMP
if nargin < 1 || ~isa(movieData,'MovieData')
    error('The first input argument must be a valid MovieData object!')
end

if nargin < 2
    paramsIn = [];
end
%Get the indices of any previous threshold processes from this function                                                                              
iProc = movieData.getProcessIndex('SpecialSegProcess',2);

%If the process doesn't exist, create it
if isempty(iProc)
    iProc = numel(movieData.processes_)+1;
    movieData.addProcess(SpecialSegProcess(movieData,movieData.outputDirectory_));                                                                                                 
end

thresProc= movieData.processes_{iProc};
%Parse input, store in parameter structure
p = parseProcessParams(movieData.processes_{iProc},paramsIn);
nChan = numel(movieData.channels_);
if max(p.ChannelIndex) > nChan || min(p.ChannelIndex)<1 || ~isequal(round(p.ChannelIndex),p.ChannelIndex)
    error('Invalid channel numbers specified! Check Channel Input Selection!')
end
nChanThresh = length(p.ChannelIndex);

%% --------------- Init ---------------%%
disp('Starting segmentation...')
% Set up the input directories (input images)
inFilePaths = cell(1,numel(movieData.channels_));
for i = p.ChannelIndex
    if isempty(p.ProcessIndex)
        inFilePaths{1,i} = movieData.getChannelPaths{i};
    else
       inFilePaths{1,i} = movieData.processes_{p.ProcessIndex}.outFilePaths_{1,i}; 
    end
end
thresProc.setInFilePaths(inFilePaths);
%Set up the mask directories as sub-directories of the output directory
for j = 1:nChanThresh;
    %Create string for current directory
    currDir = [p.OutputDirectory filesep dName num2str(p.ChannelIndex(j))];    
    %Save this in the process object
    thresProc.setOutMaskPath(p.ChannelIndex(j),currDir);
    %Check/create directory
    mkClrDir(currDir)               
end
imageFileNames = movieData.getImageFileNames(p.ChannelIndex);
nImages = movieData.nFrames_;   
nImTot = nImages * nChanThresh;
%Get mask and image directories
maskDirs  = thresProc.outFilePaths_(p.ChannelIndex);
%threshMethod = thresProc.getMethods(p.methodSelection).func;

%% Starting the process, looping through all available Channels and Images
p.BatchMode = 0; 
if ~p.BatchMode && feature('ShowFigureWindows')
    wtBar = waitbar(0,['Please wait, segmenting channel ' num2str(p.ChannelIndex(1)) ' ...']);        
else
    wtBar = -1;
end        
for iChan = 1:nChanThresh
       
    disp(['Segmenting images for channel # ' num2str(p.ChannelIndex(iChan)) ' : '])
    disp(inFilePaths{1,p.ChannelIndex(iChan)})
    disp('Masks will be stored in directory :')
    disp(maskDirs{iChan})    
    %Initialize vector for threshold values
    allThresholds{iChan} = nan(nImages,1);
    for iImage = 1:nImages
        %Load the current image
        if ishandle(wtBar)
            waitbar(iImage / nImTot,wtBar,['Please wait, segmenting channel ' num2str(p.ChannelIndex(iChan)) ' ...']);
        end
        %if isempty(p.ProcessIndex)
            currImage = movieData.channels_(p.ChannelIndex(iChan)).loadImage(iImage);
       % else
       %     currImage = movieData.processes_{p.ProcessIndex}.loadOutImage(p.ChannelIndex(iChan),iImage);
       % end

        %% nuclear segmentation

        if p.methodSelection == 1     
            [outputMask] = detectFilterPaper(currImage,'thPaper',p.thPaper);
        elseif p.methodSelection == 2
            bwPaper = movieData.processes_{p.ProcessIndex}.loadOutImage(p.ChannelIndex(iChan),iImage);
            [~,outputMask] = detectUrineSpots2(currImage,...
                'bwPaper', bwPaper,...
                'maxRectangularBoxBorderPxRatio',p.maxRect,...
                'minSpotArea', p.minSpotArea);
            %pString = [pString, 'Urine_spots_'];
        end
        imwrite(outputMask,[maskDirs{iChan} filesep pString imageFileNames{iChan}{iImage}],'tif');
    end
end

if ishandle(wtBar)
    close(wtBar)
end
thresProc.setDateTime;
movieData.save; %Save the new movieData to disk
disp('Finished segmentation!');
end